/*
  ==============================================================================

    This file contains the basic framework code for a JUCE plugin editor.

  ==============================================================================
*/

#include "PluginProcessor.h"
#include "PluginEditor.h"

//==============================================================================
TestHiDPIAudioProcessorEditor::TestHiDPIAudioProcessorEditor (TestHiDPIAudioProcessor& p)
    : AudioProcessorEditor (&p), audioProcessor (p)
{
    // Make sure that before the constructor has finished, you've set the
    // editor's size to whatever you need it to be.
    setSize (1600, 900);
}

TestHiDPIAudioProcessorEditor::~TestHiDPIAudioProcessorEditor()
{
}

//==============================================================================
void TestHiDPIAudioProcessorEditor::paint (juce::Graphics& g)
{
    // (Our component is opaque, so we must completely fill the background with a solid colour)
    g.fillAll (getLookAndFeel().findColour (juce::ResizableWindow::backgroundColourId));

	const double platformScale = getPeer ()->getPlatformScaleFactor ();
	const float approximateScale = juce::Component::getApproximateScaleFactorForComponent ( this );
	const float physicalScale = g.getInternalContext ().getPhysicalPixelScaleFactor ();

	juce::PluginHostType host;
	const bool isStandalone = juce::JUCEApplicationBase::isStandaloneApp ();

	auto text = isStandalone ? juce::String ( "Standalone" ) : juce::String ( host.getHostDescription () );
	text += juce::String ( "\n\nPlatform Scale Factor: " ) + juce::String ( platformScale );
	text += juce::String ( "\nApproximate Scale Factor: " ) + juce::String ( approximateScale );
	text += juce::String ( "\nPhysical Pixel Scale Factor: " ) + juce::String ( physicalScale );
	text += juce::String ( "\nLast Set Scale Factor: " ) + ( m_lastSetScaleFactor < 0.0f ? juce::String ( "<not set>" ) : juce::String ( m_lastSetScaleFactor ) );

	if ( m_mouseDowns.empty() == false )
		text += juce::String ( "\nLast Mouse Down Pos: " ) + juce::String ( m_mouseDowns.back ().x ) + " " + juce::String ( m_mouseDowns.back ().y );

	if ( m_mouseWheels.empty () == false )
		text += juce::String ( "\nLast Mouse Wheel Pos: " ) + juce::String ( m_mouseWheels.back ().x ) + " " + juce::String ( m_mouseWheels.back ().y );

    g.setColour (juce::Colours::white);
    g.setFont (15.0f);
    g.drawFittedText (text, getLocalBounds(), juce::Justification::centred, 1);

	for ( auto [index, point] : juce::enumerate ( m_mouseDowns ) )
	{
		g.setColour ( juce::Colours::lightblue );
		g.drawLine ( point.x - 10, point.y, point.x + 10, point.y );
		g.drawLine ( point.x, point.y - 10, point.x, point.y + 10 );
		g.drawSingleLineText ( juce::String ( "button " ) + juce::String ( index ), point.x + 12, point.y - 2 );
	}

	for ( auto [index, point] : juce::enumerate ( m_mouseWheels ) )
	{
		g.setColour ( juce::Colours::orangered );
		g.drawLine ( point.x - 7, point.y - 7, point.x + 7, point.y + 7 );
		g.drawLine ( point.x + 7, point.y - 7, point.x - 7, point.y + 7 );
		g.drawSingleLineText ( juce::String ( "wheel " ) + juce::String ( index ), point.x + 12, point.y + 12 );
	}
}

void TestHiDPIAudioProcessorEditor::resized()
{
    // This is generally where you'll want to lay out the positions of any
    // subcomponents in your editor..
}

void TestHiDPIAudioProcessorEditor::mouseDown ( const juce::MouseEvent& event )
{
	m_mouseDowns.push_back ( event.getPosition ().toFloat () );
	repaint ();
}

void TestHiDPIAudioProcessorEditor::mouseWheelMove ( const juce::MouseEvent& event, const juce::MouseWheelDetails& /*wheel*/ )
{
	if ( m_mouseWheels.empty () || m_mouseWheels.back () != event.getPosition ().toFloat () )
		m_mouseWheels.push_back ( event.getPosition ().toFloat () );

	repaint ();
}

void TestHiDPIAudioProcessorEditor::setScaleFactor ( float newScale )
{
	juce::AudioProcessorEditor::setScaleFactor ( newScale );
	m_lastSetScaleFactor = newScale;
	repaint ();
}
